@@ -2,7 +2,7 @@
 """
 A test harness for gnupg.py.
 
-Copyright (C) 2008-2016 Vinay Sajip. All rights reserved.
+Copyright (C) 2008-2017 Vinay Sajip. All rights reserved.
 """
 import doctest
 import logging
@@ -15,10 +15,20 @@ import sys
 import tempfile
 import unittest
 
+try:
+    from unittest import skipIf
+except ImportError:
+    # For now, for Python < 2.7
+    def skipIf(condition, message):
+        if not condition:
+            return lambda x: x
+        else:
+            return lambda x: None
+
 import gnupg
 
 __author__ = "Vinay Sajip"
-__date__  = "$10-Sep-2016 08:38:57$"
+__date__  = "$29-Jan-2017 17:09:59$"
 
 ALL_TESTS = True
 
@@ -147,6 +157,20 @@ def compare_keys(k1, k2):
     # ignoring things like spurious blank lines
     return get_key_data(k1) != get_key_data(k2)
 
+AGENT_CONFIG = b'''allow-loopback-pinentry
+log-file socket:///tmp/S.my-gnupg-log
+verbose
+debug ipc
+'''
+
+def prepare_homedir(hd):
+    if not os.path.isdir(hd):
+        os.makedirs(hd)
+    os.chmod(hd, 0x1C0)
+    fn = os.path.join(hd, 'gpg-agent.conf')
+    with open(fn, 'wb') as f:
+        f.write(AGENT_CONFIG)
+
 class GPGTestCase(unittest.TestCase):
     def setUp(self):
         hd = os.path.join(os.getcwd(), 'keys')
@@ -154,6 +178,7 @@ class GPGTestCase(unittest.TestCase):
             self.assertTrue(os.path.isdir(hd),
                             "Not a directory: %s" % hd)
             shutil.rmtree(hd)
+        prepare_homedir(hd)
         self.homedir = hd
         self.gpg = gpg = gnupg.GPG(gnupghome=hd, gpgbinary=GPGBINARY)
         v = gpg.version
@@ -240,6 +265,8 @@ class GPGTestCase(unittest.TestCase):
             'name_comment': 'dummy comment',
             'name_email': 'test.name@example.com',
         }
+        if self.gpg.version >= (2, 1):
+            params['passphrase'] = 'foo'
         cmd = self.gpg.gen_key_input(**params)
         result = self.gpg.gen_key(cmd)
         keys = self.gpg.list_keys()
@@ -253,10 +280,14 @@ class GPGTestCase(unittest.TestCase):
 
     def test_key_generation_with_escapes(self):
         "Test that key generation handles escape characters"
-        cmd = self.gpg.gen_key_input(name_comment='Funny chars: '
-                                                  '\\r\\n\\f\\v\\0\\b',
-                                     name_real='Test Name',
-                                     name_email='test.name@example.com')
+        params = {
+            'name_real': 'Test Name',
+            'name_comment': 'Funny chars: \\r\\n\\f\\v\\0\\b',
+            'name_email': 'test.name@example.com',
+        }
+        if self.gpg.version >= (2, 1):
+            params['passphrase'] = 'foo'
+        cmd = self.gpg.gen_key_input(**params)
         result = self.gpg.gen_key(cmd)
         keys = self.gpg.list_keys()
         self.assertEqual(len(keys), 1)
@@ -314,12 +345,19 @@ class GPGTestCase(unittest.TestCase):
                         "1-element list expected")
         self.assertEqual(len(private_keys.fingerprints), 1)
         # Now do the same test, but using keyring and secret_keyring arguments
-        pkn = 'pubring.gpg'
-        skn = 'secring.gpg'
+        if self.gpg.version < (2, 1):
+            pkn = 'pubring.gpg'
+            skn = 'secring.gpg'
+        else:
+            # On GnuPG >= 2.1, --secret-keyring is obsolete and ignored,
+            # and the keyring file name has changed.
+            pkn = 'pubring.kbx'
+            skn = None
         hd = os.path.join(os.getcwd(), 'keys')
         if os.name == 'posix':
             pkn = os.path.join(hd, pkn)
-            skn = os.path.join(hd, skn)
+            if skn:
+                skn = os.path.join(hd, skn)
         gpg = gnupg.GPG(gnupghome=hd, gpgbinary=GPGBINARY,
                         keyring=pkn, secret_keyring=skn)
         logger.debug('Using keyring and secret_keyring arguments')
@@ -377,17 +415,20 @@ class GPGTestCase(unittest.TestCase):
 
     def test_scan_keys(self):
         "Test that external key files can be scanned"
-        expected = set([
-            'Andrew Able (A test user) <andrew.able@alpha.com>',
-            'Barbara Brown (A test user) <barbara.brown@beta.com>',
-            'Charlie Clark (A test user) <charlie.clark@gamma.com>',
-        ])
-        for fn in ('test_pubring.gpg', 'test_secring.gpg'):
-            data = self.gpg.scan_keys(fn)
-            uids = set()
-            for d in data:
-                uids.add(d['uids'][0])
-            self.assertEqual(uids, expected)
+        # Don't use SkipTest for now, as not available for Python < 2.7
+        if self.gpg.version < (2, 1):
+            expected = set([
+                'Andrew Able (A test user) <andrew.able@alpha.com>',
+                'Barbara Brown (A test user) <barbara.brown@beta.com>',
+                'Charlie Clark (A test user) <charlie.clark@gamma.com>',
+            ])
+            for fn in ('test_pubring.gpg', 'test_secring.gpg'):
+                logger.debug('scanning keys in %s', fn)
+                data = self.gpg.scan_keys(fn)
+                uids = set()
+                for d in data:
+                    uids.add(d['uids'][0])
+                self.assertEqual(uids, expected)
 
     def test_encryption_and_decryption(self):
         "Test that encryption and decryption works"
@@ -455,11 +496,16 @@ class GPGTestCase(unittest.TestCase):
         self.assertEqual(0, match, "Keys must match")
         #Generate a key so we can test exporting private keys
         key = self.do_key_generation()
-        ascii = gpg.export_keys(key.fingerprint, True)
+        if self.gpg.version < (2, 1):
+            passphrase = None
+        else:
+            passphrase = 'bbrown'
+        ascii = gpg.export_keys(key.fingerprint, True, passphrase=passphrase)
         self.assertTrue(isinstance(ascii, gnupg.text_type))
         self.assertTrue(ascii.find("PGP PRIVATE KEY BLOCK") >= 0,
                         "Exported key should be private")
-        binary = gpg.export_keys(key.fingerprint, True, armor=False)
+        binary = gpg.export_keys(key.fingerprint, True, armor=False,
+                                 passphrase=passphrase)
         self.assertFalse(isinstance(binary, gnupg.text_type))
         logger.debug("test_import_and_export ends")
 
@@ -616,7 +662,7 @@ class GPGTestCase(unittest.TestCase):
         self.assertEqual(args[-4:], ['--foo', '--bar', 'a', 'b'])
 
     def do_file_encryption_and_decryption(self, encfname, decfname):
-        "Do the actual encryption.decryptin test using given filenames"
+        "Do the actual encryption/decryption test using given filenames"
         mode = None
         if os.name == 'posix':
             # pick a mode that won't be already in effect via umask
@@ -644,6 +690,7 @@ class GPGTestCase(unittest.TestCase):
             efile.seek(0, 0) # can't use os.SEEK_SET in 2.4
             edata = efile.read()
             efile.close()
+            self.assertTrue(os.path.exists(decfname))
             dfile = open(decfname, 'rb')
             ddata = dfile.read()
             dfile.close()
@@ -694,6 +741,7 @@ class GPGTestCase(unittest.TestCase):
             shutil.rmtree(d)
         logger.debug("test_filename_with_spaces ends")
 
+    #@skipIf(os.name == 'nt', 'Test not suitable for Windows')
     def test_search_keys(self):
         "Test that searching for keys works"
         r = self.gpg.search_keys('<vinay_sajip@hotmail.com>')
@@ -743,6 +791,94 @@ class GPGTestCase(unittest.TestCase):
         self.assertTrue(signed.data)
         logger.debug("test_signing_with_uid ends")
 
+    def test_doctest_import_keys(self):
+        """
+        Because GnuPG 2.1 requires passphrases for exporting and deleting
+        secret keys, and because console-mode passphrase entry requires
+        configuration changes, doctests can't always be used. This test
+        replicates the original doctest for import_keys as a regular test.
+
+        >>> import shutil
+        >>> shutil.rmtree("keys")
+        >>> GPGBINARY = os.environ.get('GPGBINARY', 'gpg')
+        >>> gpg = GPG(gpgbinary=GPGBINARY, gnupghome="keys")
+        >>> input = gpg.gen_key_input(name_email='user1@test', passphrase='pp1')
+        >>> result = gpg.gen_key(input)
+        >>> fp1 = result.fingerprint
+        >>> result = gpg.gen_key(input)
+        >>> fp2 = result.fingerprint
+        >>> pubkey1 = gpg.export_keys(fp1)
+        >>> seckey1 = gpg.export_keys(fp1, secret=True, passphrase='pp1')
+        >>> seckeys = gpg.list_keys(secret=True)
+        >>> pubkeys = gpg.list_keys()
+        >>> assert fp1 in seckeys.fingerprints
+        >>> assert fp1 in pubkeys.fingerprints
+        >>> str(gpg.delete_keys(fp1))
+        'Must delete secret key first'
+        >>> str(gpg.delete_keys(fp1, secret=True, passphrase='pp1'))
+        'ok'
+        >>> str(gpg.delete_keys(fp1))
+        'ok'
+        >>> str(gpg.delete_keys("nosuchkey"))
+        'No such key'
+        >>> seckeys = gpg.list_keys(secret=True)
+        >>> pubkeys = gpg.list_keys()
+        >>> assert not fp1 in seckeys.fingerprints
+        >>> assert not fp1 in pubkeys.fingerprints
+        >>> result = gpg.import_keys('foo')
+        >>> assert not result
+        >>> result = gpg.import_keys(pubkey1)
+        >>> pubkeys = gpg.list_keys()
+        >>> seckeys = gpg.list_keys(secret=True)
+        >>> assert not fp1 in seckeys.fingerprints
+        >>> assert fp1 in pubkeys.fingerprints
+        >>> result = gpg.import_keys(seckey1)
+        >>> assert result
+        >>> seckeys = gpg.list_keys(secret=True)
+        >>> pubkeys = gpg.list_keys()
+        >>> assert fp1 in seckeys.fingerprints
+        >>> assert fp1 in pubkeys.fingerprints
+        >>> assert fp2 in pubkeys.fingerprints
+        """
+        logger.debug("test_doctest_import_keys begins")
+        gpg = self.gpg
+        inp = gpg.gen_key_input(name_email='user1@test', passphrase='pp1')
+        result = gpg.gen_key(inp)
+        fp1 = result.fingerprint
+        inp = gpg.gen_key_input(name_email='user2@test', passphrase='pp2')
+        result = gpg.gen_key(inp)
+        fp2 = result.fingerprint
+        pubkey1 = gpg.export_keys(fp1)
+        if gpg.version >= (2, 1):
+            passphrase = 'pp1'
+        else:
+            passphrase = None
+        seckey1 = gpg.export_keys(fp1, secret=True, passphrase=passphrase)
+        seckeys = gpg.list_keys(secret=True)
+        pubkeys = gpg.list_keys()
+        # avoid assertIn, etc. as absent in older Python versions
+        self.assertTrue(fp1 in seckeys.fingerprints)
+        self.assertTrue(fp1 in pubkeys.fingerprints)
+        result = gpg.delete_keys(fp1)
+        self.assertEqual(str(result), 'Must delete secret key first')
+        if gpg.version < (2, 1):
+            # Doesn't work on 2.1, and can't use SkipTest due to having
+            # to support older Pythons
+            result = gpg.delete_keys(fp1, secret=True, passphrase=passphrase)
+            self.assertEqual(str(result), 'ok')
+            result = gpg.delete_keys(fp1)
+            self.assertEqual(str(result), 'ok')
+            result = gpg.delete_keys('nosuchkey')
+            self.assertEqual(str(result), 'No such key')
+            seckeys = gpg.list_keys(secret=True)
+            pubkeys = gpg.list_keys()
+            self.assertFalse(fp1 in seckeys.fingerprints)
+            self.assertFalse(fp1 in pubkeys.fingerprints)
+            result = gpg.import_keys('foo')
+            self.assertFalse(result)
+        logger.debug("test_doctest_import_keys ends")
+
+
 TEST_GROUPS = {
     'sign' : set(['test_signature_verification']),
     'crypt' : set(['test_encryption_and_decryption',
@@ -756,11 +892,11 @@ TEST_GROUPS = {
                  'test_key_generation_with_empty_value',
                  'test_key_generation_with_colons',
                  'test_search_keys', 'test_scan_keys']),
-    'import' : set(['test_import_only']),
+    'import' : set(['test_import_only', 'test_doctest_import_keys']),
     'basic' : set(['test_environment', 'test_list_keys_initial',
                    'test_nogpg', 'test_make_args',
                    'test_quote_with_shell']),
-    'test': set(['test_search_keys']),
+    'test': set(['test_doctest_import_keys']),
 }
 
 def suite(args=None):
@@ -791,7 +927,7 @@ def init_logging():
 def main():
     init_logging()
     tests = suite()
-    results = unittest.TextTestRunner(verbosity=2).run(tests)
+    results = unittest.TextTestRunner(verbosity=1).run(tests)
     return not results.wasSuccessful()
 
 
